@@ -12,4 +12,9 @@ __PACKAGE__->config('name' => 'TestApp');
 
 __PACKAGE__->setup;
 
+sub handle_unicode_encoding_exception {
+  my ( $self, $param_value, $error_msg ) = @_;
+  return $param_value;
+}
+
 1;
